import sys

from src.roadmapper.roadmap import Roadmap
from src.roadmapper.timelinemode import TimelineMode
from src.tests.roadmap_generators.roadmap_abc import RoadmapABC


class ColourTheme(RoadmapABC):

    def generate_and_save_as(
            self,
            file_name: str = "example.png",
            colour_theme: str = "DEFAULT",
    ) -> None:
        roadmap: Roadmap = self.generate()
        roadmap.draw()
        roadmap.save(file_name)

    def generate(self, colour_theme: str = "DEFAULT") -> Roadmap:
        width = 1200
        height = 1000
        auto_height = True
        mode = TimelineMode.MONTHLY
        start_date = "2023-01-01"
        number_of_items = 12
        show_generic_dates = False

        roadmap = Roadmap(width, height, auto_height=auto_height, colour_theme=colour_theme, show_marker=False)

        roadmap.set_title("SAMPLE ROADMAP 2022/2023")
        roadmap.set_subtitle("GodZone Corporation")

        roadmap.set_timeline(
            mode=mode,
            start=start_date,
            number_of_items=number_of_items,
            show_generic_dates=show_generic_dates,
        )

        group = roadmap.add_group("Core Product Work Stream", text_alignment="left")
        task = group.add_task(
            "Base Functionality", "2022-11-01", "2023-10-31"
        )
        task.add_milestone(
            "v.1.0", "2023-02-15"
        )
        task.add_milestone(
            "v.1.1", "2023-08-01"
        )
        parellel_task = task.add_parallel_task(
            "Enhancements", "2023-11-15", "2024-03-31"
        )
        parellel_task.add_milestone(
            "v.2.0", "2024-03-30"
        )
        task = group.add_task(
            "Showcase #1", "2023-03-01", "2023-05-07"
        )
        task.add_parallel_task(
            "Showcase #2", "2023-06-01", "2023-08-07"
        )

        group = roadmap.add_group("Mobility Work Stream", text_alignment="left")
        group.add_task(
            "Mobile App Development", "2023-02-01", "2024-12-07"
        )

        roadmap.set_footer("Generated by Roadmapper")

        return roadmap


def is_arg_given():
    return len(sys.argv) > 1


if __name__ == '__main__':
    if is_arg_given():
        example_name = sys.argv[1]
        ColourTheme().generate_and_save_as(file_name=example_name)
    else:
        ColourTheme.generate_and_save_as()
